﻿:namespace  inout

    ⎕IO←1 ⋄ ⎕ML←0

    ⍝ There are 2 possibilities:
    ⍝ 1. we are dealing with ATF files (extension .ATF)     : in / out
    ⍝ 2. we are dealing with Dyalog's (own) eXtended format : inx/ outx

    Cmds←'in' 'out' 'inx' 'outx'

    ⍝ We could have created only 2 cmds (in & out) to avoid having another set of 2 but because
    ⍝ their switches are different it was written this way.

    ∇ r←List
      :Access Shared Public
      r←⎕NS¨(⍴Cmds)⍴⊂''
      r.Name←Cmds
      r[1].Desc←'Define an ATF file''s objects in the ws' ⍝ descriptions
      r[2].Desc←'Create an ATF file from the current ws'
      r[3].Desc←'Define in the ws the objects of an extended transfer file'
      r[4].Desc←'Create an extended transfer file from the current ws'
      r.Group←⊂'Transfer'                                 ⍝ same group
     ⍝ Parsing rules for each:
      r[1].Parse←'1 -fliplu -apl='
      r[2].Parse←'1 -obj='
      r[3].Parse←'1 -apl= -file[=] -list -noam -obj= -range= -replace -trans[=]0 1 2'
      r[4].Parse←'1 -obj= -file[=] -range= -lock='
    ∇

    ∇ r←Run(Cmd Args);arg;b;q
      :Access Shared Public
      :If 'x'∊Cmd
          ##.⎕CY'xfrcode'
      :Else
          ⎕FIX ⎕SE.SALT.Load'lib\atfin -source'
      :EndIf
      :If ∨/b←'''" -'∊arg←1⊃Args.Arguments
          ⍝ The fns below will reparse the line so we need to surround the arg properly with quotes
          arg←q,((1+q=arg)/arg),q←''''
      :End
      :Select Cmd
      :Case 1⊃Cmds
          ⍙⍙.ATFIN arg,Args.Propagate'fliplu apl'
      :Case 2⊃Cmds
          ⍙⍙.ATFOUT arg,Args.Propagate'obj'
      :Case 3⊃Cmds
          ##.THIS ##.xfr.∆xfrfrom arg,Args.Propagate'apl file list noam obj range replace trans'
      :Case 4⊃Cmds
          ##.xfr.∆xfrto arg,Args.Propagate'file range lock obj'
      :EndSelect
      r←0 0⍴''
    ∇

    ∇ r←lev Help Cmd;o;i;ix;ox
      :Access Shared Public
      :If Cmd≡'in'
          i←⊂'Bring objects defined in an ATF file.'
          i,←⊂'The source APL can be specified through the -apl switch. If the source APL was using'
          i,←⊂'underscored letters you can replace them by lowercase (switch them) by using the -fliplu'
          i,←⊂'switch.'
          r←↑i ⋄ →lev↓0
          r←↑i,'' 'Example: bring in an ATF file generated by APL+:' '      ]in \tmp\aplp'
      :ElseIf Cmd≡'out'
          o←⊂'Write objects, vars and fns only (default all) in current ns, to an ATF file.'
          o,←⊂'You can specify which objects to be written out with the -obj switch'
          r←↑o ⋄ →lev↓0
          r←↑o,'' 'Example: bring in an ATF file generated by APL+:' '      ]in \tmp\aplp'
      :ElseIf Cmd≡'inx'
          ix←⊂'Bring objects defined in an extended transfer file'
          ix,←⊂'Several switches allow to fine tune the process. Use ]??inx for details.'
          r←↑ix ⋄ →lev↓0
          'xfr.∆describe'⎕CY'xfrcode'
          r←∆describe
      :ElseIf Cmd≡'outx'
          ox←⊂'Write objects (default all in current ws) to an extended transfer file.'
          ox,←⊂'Specific objects can be selected thru the -obj switch. Use ]??outx for details.'
          r←↑ox ⋄ →lev↓0
          'xfr.∆describe'⎕CY'xfrcode'
          r←∆describe
      :EndIf
    ∇

:Endnamespace